@@ -1,24 +1,39 @@
+from collections import defaultdict
+from azure.durable_functions.models.actions.SignalEntityAction import SignalEntityAction
+from azure.durable_functions.models.actions.CallEntityAction import CallEntityAction
+from azure.durable_functions.models.Task import TaskBase
+from azure.durable_functions.models.actions.CallHttpAction import CallHttpAction
+from azure.durable_functions.models.DurableHttpRequest import DurableHttpRequest
+from azure.durable_functions.models.actions.CallSubOrchestratorWithRetryAction import \
+    CallSubOrchestratorWithRetryAction
+from azure.durable_functions.models.actions.CallActivityWithRetryAction import \
+    CallActivityWithRetryAction
+from azure.durable_functions.models.actions.ContinueAsNewAction import \
+    ContinueAsNewAction
+from azure.durable_functions.models.actions.WaitForExternalEventAction import \
+    WaitForExternalEventAction
+from azure.durable_functions.models.actions.CallSubOrchestratorAction import \
+    CallSubOrchestratorAction
+from azure.durable_functions.models.actions.CreateTimerAction import CreateTimerAction
+from azure.durable_functions.models.Task import WhenAllTask, WhenAnyTask, AtomicTask, \
+    RetryAbleTask
+from azure.durable_functions.models.actions.CallActivityAction import CallActivityAction
 from azure.durable_functions.models.ReplaySchema import ReplaySchema
 import json
 import datetime
 import inspect
-from typing import List, Any, Dict, Optional
-from uuid import UUID, uuid5, NAMESPACE_URL
+from typing import DefaultDict, List, Any, Dict, Optional, Tuple, Union
+from uuid import UUID, uuid5, NAMESPACE_URL, NAMESPACE_OID
 from datetime import timezone
 
 from .RetryOptions import RetryOptions
-from .TaskSet import TaskSet
 from .FunctionContext import FunctionContext
 from .history import HistoryEvent, HistoryEventType
 from .actions import Action
-from ..models.Task import Task
 from ..models.TokenSource import TokenSource
 from .utils.entity_utils import EntityId
-from ..tasks import call_activity_task, task_all, task_any, call_activity_with_retry_task, \
-    wait_for_external_event_task, continue_as_new, new_uuid, call_http, create_timer_task, \
-    call_sub_orchestrator_task, call_sub_orchestrator_with_retry_task, call_entity_task, \
-    signal_entity_task
 from azure.functions._durable_functions import _deserialize_custom_object
+from azure.durable_functions.constants import DATETIME_STRING_FORMAT
 
 
 class DurableOrchestrationContext:
@@ -47,16 +62,21 @@ class DurableOrchestrationContext:
             self.decision_started_event.timestamp
         self._new_uuid_counter = 0
         self._function_context: FunctionContext = FunctionContext(**kwargs)
+        self._sequence_number = 0
         self._replay_schema = ReplaySchema(upperSchemaVersion)
-        self.actions: List[List[Action]] = []
-        if self._replay_schema == ReplaySchema.V2:
-            self.actions.append([])
+
+        self._action_payload_v1: List[List[Action]] = []
+        self._action_payload_v2: List[Action] = []
 
         # make _input always a string
         # (consistent with Python Functions generic trigger/input bindings)
         if (isinstance(input, Dict)):
             input = json.dumps(input)
+
         self._input: Any = input
+        self.open_tasks: DefaultDict[Union[int, str], Union[List[TaskBase], TaskBase]]
+        self.open_tasks = defaultdict(list)
+        self.deferred_tasks: Dict[Union[int, str], Tuple[HistoryEvent, bool, str]] = {}
 
     @classmethod
     def from_json(cls, json_string: str):
@@ -73,11 +93,56 @@ class DurableOrchestrationContext:
             New instance of the durable orchestration context class
         """
         # We should consider parsing the `Input` field here as well,
-        # intead of doing so lazily when `get_input` is called.
+        # instead of doing so lazily when `get_input` is called.
         json_dict = json.loads(json_string)
         return cls(**json_dict)
 
-    def call_activity(self, name: str, input_: Optional[Any] = None) -> Task:
+    def _generate_task(self, action: Action,
+                       retry_options: Optional[RetryOptions] = None,
+                       id_: Optional[Union[int, str]] = None,
+                       parent: Optional[TaskBase] = None) -> Union[AtomicTask, RetryAbleTask]:
+        """Generate an atomic or retryable Task based on an input.
+
+        Parameters
+        ----------
+        action : Action
+            The action backing the Task.
+        retry_options : Optional[RetryOptions]
+            RetryOptions for a with-retry task, by default None
+
+        Returns
+        -------
+        Union[AtomicTask, RetryAbleTask]
+            Either an atomic task or a retry-able task
+        """
+        # Create an atomic task
+        task: Union[AtomicTask, RetryAbleTask]
+        action_payload: Union[Action, List[Action]]
+
+        # TODO: find cleanear way to do this
+        if self._replay_schema is ReplaySchema.V1:
+            action_payload = [action]
+        else:
+            action_payload = action
+        task = AtomicTask(id_, action_payload)
+        task.parent = parent
+
+        # if task is retryable, provide the retryable wrapper class
+        if not(retry_options is None):
+            task = RetryAbleTask(task, retry_options, self)
+        return task
+
+    def _set_is_replaying(self, is_replaying: bool):
+        """Set the internal `is_replaying` flag.
+
+        Parameters
+        ----------
+        is_replaying : bool
+            New value of the `is_replaying` flag
+        """
+        self._is_replaying = is_replaying
+
+    def call_activity(self, name: str, input_: Optional[Any] = None) -> TaskBase:
         """Schedule an activity for execution.
 
         Parameters
@@ -92,14 +157,13 @@ class DurableOrchestrationContext:
         Task
             A Durable Task that completes when the called activity function completes or fails.
         """
-        return call_activity_task(
-            state=self.histories,
-            name=name,
-            input_=input_)
+        action = CallActivityAction(name, input_)
+        task = self._generate_task(action)
+        return task
 
     def call_activity_with_retry(self,
                                  name: str, retry_options: RetryOptions,
-                                 input_: Optional[Any] = None) -> Task:
+                                 input_: Optional[Any] = None) -> TaskBase:
         """Schedule an activity for execution with retry options.
 
         Parameters
@@ -117,15 +181,13 @@ class DurableOrchestrationContext:
             A Durable Task that completes when the called activity function completes or
             fails completely.
         """
-        return call_activity_with_retry_task(
-            state=self.histories,
-            retry_options=retry_options,
-            name=name,
-            input_=input_)
+        action = CallActivityWithRetryAction(name, retry_options, input_)
+        task = self._generate_task(action, retry_options)
+        return task
 
     def call_http(self, method: str, uri: str, content: Optional[str] = None,
                   headers: Optional[Dict[str, str]] = None,
-                  token_source: TokenSource = None) -> Task:
+                  token_source: TokenSource = None) -> TaskBase:
         """Schedule a durable HTTP call to the specified endpoint.
 
         Parameters
@@ -146,13 +208,20 @@ class DurableOrchestrationContext:
         Task
             The durable HTTP request to schedule.
         """
-        return call_http(
-            state=self.histories, method=method, uri=uri, content=content, headers=headers,
-            token_source=token_source)
+        json_content: Optional[str] = None
+        if content and content is not isinstance(content, str):
+            json_content = json.dumps(content)
+        else:
+            json_content = content
+
+        request = DurableHttpRequest(method, uri, json_content, headers, token_source)
+        action = CallHttpAction(request)
+        task = self._generate_task(action)
+        return task
 
     def call_sub_orchestrator(self,
                               name: str, input_: Optional[Any] = None,
-                              instance_id: Optional[str] = None) -> Task:
+                              instance_id: Optional[str] = None) -> TaskBase:
         """Schedule sub-orchestration function named `name` for execution.
 
         Parameters
@@ -169,17 +238,14 @@ class DurableOrchestrationContext:
         Task
             A Durable Task that completes when the called sub-orchestrator completes or fails.
         """
-        return call_sub_orchestrator_task(
-            context=self,
-            state=self.histories,
-            name=name,
-            input_=input_,
-            instance_id=instance_id)
+        action = CallSubOrchestratorAction(name, input_, instance_id)
+        task = self._generate_task(action)
+        return task
 
     def call_sub_orchestrator_with_retry(self,
                                          name: str, retry_options: RetryOptions,
                                          input_: Optional[Any] = None,
-                                         instance_id: Optional[str] = None) -> Task:
+                                         instance_id: Optional[str] = None) -> TaskBase:
         """Schedule sub-orchestration function named `name` for execution, with retry-options.
 
         Parameters
@@ -198,13 +264,9 @@ class DurableOrchestrationContext:
         Task
             A Durable Task that completes when the called sub-orchestrator completes or fails.
         """
-        return call_sub_orchestrator_with_retry_task(
-            context=self,
-            state=self.histories,
-            retry_options=retry_options,
-            name=name,
-            input_=input_,
-            instance_id=instance_id)
+        action = CallSubOrchestratorWithRetryAction(name, retry_options, input_, instance_id)
+        task = self._generate_task(action, retry_options)
+        return task
 
     def get_input(self) -> Optional[Any]:
         """Get the orchestration input."""
@@ -224,9 +286,17 @@ class DurableOrchestrationContext:
         str
             New UUID that is safe for replay within an orchestration or operation.
         """
-        return new_uuid(context=self)
+        URL_NAMESPACE: str = "9e952958-5e33-4daf-827f-2fa12937b875"
 
-    def task_all(self, activities: List[Task]) -> TaskSet:
+        uuid_name_value = \
+            f"{self._instance_id}" \
+            f"_{self.current_utc_datetime.strftime(DATETIME_STRING_FORMAT)}" \
+            f"_{self._new_uuid_counter}"
+        self._new_uuid_counter += 1
+        namespace_uuid = uuid5(NAMESPACE_OID, URL_NAMESPACE)
+        return str(uuid5(namespace_uuid, uuid_name_value))
+
+    def task_all(self, activities: List[TaskBase]) -> TaskBase:
         """Schedule the execution of all activities.
 
         Similar to Promise.all. When called with `yield` or `return`, returns an
@@ -244,9 +314,9 @@ class DurableOrchestrationContext:
         TaskSet
             The results of all activities.
         """
-        return task_all(tasks=activities, replay_schema=self._replay_schema)
+        return WhenAllTask(activities, replay_schema=self._replay_schema)
 
-    def task_any(self, activities: List[Task]) -> TaskSet:
+    def task_any(self, activities: List[TaskBase]) -> TaskBase:
         """Schedule the execution of all activities.
 
         Similar to Promise.race. When called with `yield` or `return`, returns
@@ -264,7 +334,7 @@ class DurableOrchestrationContext:
         TaskSet
             The first [[Task]] instance to complete.
         """
-        return task_any(tasks=activities, replay_schema=self._replay_schema)
+        return WhenAnyTask(activities, replay_schema=self._replay_schema)
 
     def set_custom_status(self, status: Any):
         """Set the customized orchestration status for your orchestrator function.
@@ -385,7 +455,25 @@ class DurableOrchestrationContext:
         Task
             A Task of the entity call
         """
-        return call_entity_task(self.histories, entityId, operationName, operationInput)
+        action = CallEntityAction(entityId, operationName, operationInput)
+        task = self._generate_task(action)
+        return task
+
+    def _record_fire_and_forget_action(self, action: Action):
+        """Append a responseless-API action object to the actions array.
+
+        Parameters
+        ----------
+        action : Action
+            The action to append
+        """
+        new_action: Union[List[Action], Action]
+        if self._replay_schema is ReplaySchema.V2:
+            new_action = action
+        else:
+            new_action = [action]
+        self._add_to_actions(new_action)
+        self._sequence_number += 1
 
     def signal_entity(self, entityId: EntityId,
                       operationName: str, operationInput: Optional[Any] = None):
@@ -405,14 +493,17 @@ class DurableOrchestrationContext:
         Task
             A Task of the entity signal
         """
-        return signal_entity_task(self, self.histories, entityId, operationName, operationInput)
+        action = SignalEntityAction(entityId, operationName, operationInput)
+        task = self._generate_task(action)
+        self._record_fire_and_forget_action(action)
+        return task
 
     @property
     def will_continue_as_new(self) -> bool:
         """Return true if continue_as_new was called."""
         return self._continue_as_new_flag
 
-    def create_timer(self, fire_at: datetime.datetime) -> Task:
+    def create_timer(self, fire_at: datetime.datetime) -> TaskBase:
         """Create a Durable Timer Task to implement a deadline at which to wake-up the orchestrator.
 
         Parameters
@@ -422,12 +513,14 @@ class DurableOrchestrationContext:
 
         Returns
         -------
-        TimerTask
+        TaskBase
             A Durable Timer Task that schedules the timer to wake up the activity
         """
-        return create_timer_task(state=self.histories, fire_at=fire_at)
+        action = CreateTimerAction(fire_at)
+        task = self._generate_task(action)
+        return task
 
-    def wait_for_external_event(self, name: str) -> Task:
+    def wait_for_external_event(self, name: str) -> TaskBase:
         """Wait asynchronously for an event to be raised with the name `name`.
 
         Parameters
@@ -440,7 +533,9 @@ class DurableOrchestrationContext:
         Task
             Task to wait for the event
         """
-        return wait_for_external_event_task(state=self.histories, name=name)
+        action = WaitForExternalEventAction(name)
+        task = self._generate_task(action, id_=name)
+        return task
 
     def continue_as_new(self, input_: Any):
         """Schedule the orchestrator to continue as new.
@@ -450,7 +545,9 @@ class DurableOrchestrationContext:
         input_ : Any
             The new starting input to the orchestrator.
         """
-        return continue_as_new(context=self, input_=input_)
+        continue_as_new_action: Action = ContinueAsNewAction(input_)
+        self._record_fire_and_forget_action(continue_as_new_action)
+        self._continue_as_new_flag = True
 
     def new_guid(self) -> UUID:
         """Generate a replay-safe GUID.
@@ -466,6 +563,41 @@ class DurableOrchestrationContext:
         guid = uuid5(NAMESPACE_URL, guid_name)
         return guid
 
+    @property
+    def _actions(self) -> List[List[Action]]:
+        """Get the actions payload of this context, for replay in the extension.
+
+        Returns
+        -------
+        List[List[Action]]
+            The actions of this context
+        """
+        if self._replay_schema is ReplaySchema.V1:
+            return self._action_payload_v1
+        else:
+            return [self._action_payload_v2]
+
+    def _add_to_actions(self, action_repr: Union[List[Action], Action]):
+        """Add a Task's actions payload to the context's actions array.
+
+        Parameters
+        ----------
+        action_repr : Union[List[Action], Action]
+            The tasks to add
+        """
+        # Do not add further actions after `continue_as_new` has been
+        # called
+        if self.will_continue_as_new:
+            return
+
+        if self._replay_schema is ReplaySchema.V1 and isinstance(action_repr, list):
+            self._action_payload_v1.append(action_repr)
+        elif self._replay_schema is ReplaySchema.V2 and isinstance(action_repr, Action):
+            self._action_payload_v2.append(action_repr)
+        else:
+            raise Exception(f"DF-internal exception: ActionRepr of signature {type(action_repr)}"
+                            f"is not compatible on ReplaySchema {self._replay_schema.name}. ")
+
     def _pretty_print_history(self) -> str:
         """Get a pretty-printed version of the orchestration's internal history."""
         def history_to_string(event):
@@ -477,3 +609,20 @@ class DurableOrchestrationContext:
                     json_dict[key] = val
             return json.dumps(json_dict)
         return str(list(map(history_to_string, self._histories)))
+
+    def _add_to_open_tasks(self, task: TaskBase):
+
+        if isinstance(task, AtomicTask):
+            if task.id is None:
+                task.id = self._sequence_number
+                self._sequence_number += 1
+                self.open_tasks[task.id] = task
+            elif task.id != -1:
+                self.open_tasks[task.id].append(task)
+
+            if task.id in self.deferred_tasks:
+                task_update_action = self.deferred_tasks[task.id]
+                task_update_action()
+        else:
+            for child in task.children:
+                self._add_to_open_tasks(child)
