@@ -150,7 +150,7 @@ def add_call_entity_action_for_entity(state: OrchestratorState, id_: df.EntityId
 
 
 def base_expected_state(output=None, replay_schema: ReplaySchema = ReplaySchema.V1) -> OrchestratorState:
-    return OrchestratorState(is_done=False, actions=[], output=output, replay_schema=replay_schema.V1.value)
+    return OrchestratorState(is_done=False, actions=[], output=output, replay_schema=replay_schema.value)
 
 def add_call_entity_action(state: OrchestratorState, id_: df.EntityId, op: str, input_: Any):
     action = CallEntityAction(entity_id=id_, operation=op, input_=input_)
@@ -161,8 +161,8 @@ def add_signal_entity_action(state: OrchestratorState, id_: df.EntityId, op: str
     state.actions.append([action])
 
 def add_call_entity_completed_events(
-        context_builder: ContextBuilder, op: str, instance_id=str, input_=None):
-    context_builder.add_event_sent_event(instance_id)
+        context_builder: ContextBuilder, op: str, instance_id=str, input_=None, event_id=0):
+    context_builder.add_event_sent_event(instance_id, event_id)
     context_builder.add_orchestrator_completed_event()
     context_builder.add_orchestrator_started_event()
     context_builder.add_event_raised_event(name="0000", id_=0, input_=input_, is_entity=True)
@@ -196,11 +196,29 @@ def test_signal_entity_sent():
     #assert_valid_schema(result)
     assert_orchestration_state_equals(expected, result)
 
+def test_signal_entity_sent_and_response_received():
+    entityId = df.EntityId("Counter", "myCounter")
+    context_builder = ContextBuilder('test_simple_function')
+    add_call_entity_completed_events(context_builder, "get", df.EntityId.get_scheduler_id(entityId), 3, 1)
+
+
+    result = get_orchestration_state_result(
+        context_builder, generator_function_signal_entity)
+
+    expected_state = base_expected_state([3])
+    add_signal_entity_action(expected_state, entityId, "add", 3)
+    add_call_entity_action(expected_state, entityId, "get", None)
+    expected_state._is_done = True
+    expected = expected_state.to_json()
+
+    #assert_valid_schema(result)
+    assert_orchestration_state_equals(expected, result)
+
 
 def test_call_entity_raised():
     entityId = df.EntityId("Counter", "myCounter")
     context_builder = ContextBuilder('test_simple_function')
-    add_call_entity_completed_events(context_builder, "add", df.EntityId.get_scheduler_id(entityId), 3)
+    add_call_entity_completed_events(context_builder, "add", df.EntityId.get_scheduler_id(entityId), 3, 0)
 
     result = get_orchestration_state_result(
         context_builder, generator_function_call_entity)
